@@ -38,10 +38,28 @@
 
 using namespace Tiled;
 
-QSize OrthogonalRenderer::mapSize() const
+QRect OrthogonalRenderer::mapBoundingRect() const
 {
-    return QSize(map()->width() * map()->tileWidth(),
-                 map()->height() * map()->tileHeight());
+    if (!map()->infinite()) {
+        return QRect(0, 0, map()->width() * map()->tileWidth(),
+                     map()->height() * map()->tileHeight());
+    }
+
+    QRect mapBounds;
+
+    LayerIterator iterator(map());
+    while (Layer *layer = iterator.next()) {
+        if (TileLayer *tileLayer = dynamic_cast<TileLayer*>(layer))
+            mapBounds = mapBounds.united(tileLayer->bounds());
+    }
+
+    if (mapBounds.size() == QSize(0, 0))
+        mapBounds.setSize(QSize(1, 1));
+
+    return QRect(mapBounds.x() * map()->tileWidth(),
+                 mapBounds.y() * map()->tileHeight(),
+                 mapBounds.width() * map()->tileWidth(),
+                 mapBounds.height() * map()->tileHeight());
 }
 
 QRect OrthogonalRenderer::boundingRect(const QRect &rect) const
@@ -93,13 +111,21 @@ QRectF OrthogonalRenderer::boundingRect(const MapObject *object) const
                                                10 + extraSpace + 1,
                                                10 + extraSpace + 1);
             } else {
-                boundingRect = bounds.adjusted(-extraSpace,
-                                               -extraSpace,
-                                               extraSpace + 1,
-                                               extraSpace + 1);
+            boundingRect = bounds.adjusted(-extraSpace,
+                                           -extraSpace,
+                                           extraSpace + 1,
+                                           extraSpace + 1);
             }
             break;
 
+        case MapObject::Point:
+            boundingRect = shape(object).boundingRect()
+                           .adjusted(-extraSpace,
+                                     -extraSpace,
+                                     extraSpace + 1,
+                                     extraSpace + 1);
+            break;
+
         case MapObject::Polygon:
         case MapObject::Polyline: {
             // Make some more room for the starting dot
@@ -169,6 +195,10 @@ QPainterPath OrthogonalRenderer::shape(const MapObject *object) const
             break;
         }
 
+        case MapObject::Point: {
+            path = pointShape(object);
+            break;
+        }
         case MapObject::Text: {
             path.addRect(object->bounds());
             break;
@@ -188,12 +218,17 @@ void OrthogonalRenderer::drawGrid(QPainter *painter, const QRectF &rect,
     if (tileWidth <= 0 || tileHeight <= 0)
         return;
 
-    const int startX = qMax(0, (int) (rect.x() / tileWidth) * tileWidth);
-    const int startY = qMax(0, (int) (rect.y() / tileHeight) * tileHeight);
-    const int endX = qMin(qCeil(rect.right()),
-                          map()->width() * tileWidth + 1);
-    const int endY = qMin(qCeil(rect.bottom()),
-                          map()->height() * tileHeight + 1);
+    int startX = qFloor(rect.x() / tileWidth) * tileWidth;
+    int startY = qFloor(rect.y() / tileHeight) * tileHeight;
+    int endX = qCeil(rect.right());
+    int endY = qCeil(rect.bottom());
+
+    if (!map()->infinite()) {
+        startX = qMax(0, startX);
+        startY = qMax(0, startY);
+        endX = qMin(endX, map()->width() * tileWidth + 1);
+        endY = qMin(endY, map()->height() * tileHeight + 1);
+    }
 
     QPen gridPen = makeGridPen(painter->device(), gridColor);
 
@@ -225,10 +260,11 @@ void OrthogonalRenderer::drawTileLayer(QPainter *painter,
     const QPointF layerPos(layer->x() * tileWidth,
                            layer->y() * tileHeight);
 
-    int startX = 0;
-    int startY = 0;
-    int endX = layer->width() - 1;
-    int endY = layer->height() - 1;
+    QRect bounds = layer->bounds().translated(-layer->position());
+    int startX = bounds.left();
+    int startY = bounds.top();
+    int endX = bounds.right();
+    int endY = bounds.bottom();
 
     if (!exposed.isNull()) {
         QMargins drawMargins = layer->drawMargins();
@@ -242,8 +278,8 @@ void OrthogonalRenderer::drawTileLayer(QPainter *painter,
 
         rect.translate(-layerPos);
 
-        startX = qMax(qFloor(rect.x() / tileWidth), 0);
-        startY = qMax(qFloor(rect.y() / tileHeight), 0);
+        startX = qMax(qFloor(rect.x() / tileWidth), startX);
+        startY = qMax(qFloor(rect.y() / tileHeight), startY);
         endX = qMin(qCeil(rect.right()) / tileWidth, endX);
         endY = qMin(qCeil(rect.bottom()) / tileHeight, endY);
     }
@@ -276,7 +312,6 @@ void OrthogonalRenderer::drawTileLayer(QPainter *painter,
         incY = -1;
         break;
     case Map::RightDown:
-    default:
         break;
     }
 
@@ -458,6 +493,10 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
             painter->drawText(rect, textData.text, textData.textOption());
             break;
         }
+        case MapObject::Point: {
+            drawPointObject(painter, color);
+            break;
+        }
         }
     }
 
