@@ -37,8 +37,9 @@ Tile::Tile(int id, Tileset *tileset):
     Object(TileType),
     mId(id),
     mTileset(tileset),
+    mImageStatus(LoadingReady),
     mTerrain(-1),
-    mProbability(1.f),
+    mProbability(1.0),
     mObjectGroup(nullptr),
     mCurrentFrameIndex(0),
     mUnusedTime(0)
@@ -49,8 +50,9 @@ Tile::Tile(const QPixmap &image, int id, Tileset *tileset):
     mId(id),
     mTileset(tileset),
     mImage(image),
+    mImageStatus(image.isNull() ? LoadingError : LoadingReady),
     mTerrain(-1),
-    mProbability(1.f),
+    mProbability(1.0),
     mObjectGroup(nullptr),
     mCurrentFrameIndex(0),
     mUnusedTime(0)
