@@ -36,6 +36,8 @@
 
 #include <QImage>
 
+#include "qtcompat_p.h"
+
 namespace Tiled {
 
 TilesetManager *TilesetManager::mInstance;
@@ -63,8 +65,7 @@ TilesetManager::TilesetManager():
 
 TilesetManager::~TilesetManager()
 {
-    // Since all MapDocuments should be deleted first, we assert that there are
-    // no remaining tileset references.
+    // Assert that there are no remaining tileset instances
     Q_ASSERT(mTilesets.isEmpty());
 }
 
@@ -111,13 +112,9 @@ SharedTileset TilesetManager::loadTileset(const QString &fileName, QString *erro
  */
 SharedTileset TilesetManager::findTileset(const QString &fileName) const
 {
-    QMapIterator<SharedTileset, int> it(mTilesets);
-
-    while (it.hasNext()) {
-        const SharedTileset &tileset = it.next().key();
+    for (Tileset *tileset : mTilesets)
         if (tileset->fileName() == fileName)
-            return tileset;
-    }
+            return tileset->sharedPointer();
 
     return SharedTileset();
 }
@@ -126,68 +123,43 @@ SharedTileset TilesetManager::findTileset(const QString &fileName) const
  * Adds a tileset reference. This will make sure the tileset is watched for
  * changes and can be found using findTileset().
  */
-void TilesetManager::addReference(const SharedTileset &tileset)
+void TilesetManager::addTileset(Tileset *tileset)
 {
-    if (mTilesets.contains(tileset)) {
-        mTilesets[tileset]++;
-    } else {
-        mTilesets.insert(tileset, 1);
-        if (!tileset->imageSource().isEmpty())
-            mWatcher->addPath(tileset->imageSource());
-    }
+    Q_ASSERT(!mTilesets.contains(tileset));
+    mTilesets.append(tileset);
 }
 
 /**
  * Removes a tileset reference. When the last reference has been removed,
  * the tileset is no longer watched for changes.
  */
-void TilesetManager::removeReference(const SharedTileset &tileset)
+void TilesetManager::removeTileset(Tileset *tileset)
 {
-    Q_ASSERT(mTilesets.value(tileset) > 0);
-    mTilesets[tileset]--;
+    Q_ASSERT(mTilesets.contains(tileset));
+    mTilesets.removeOne(tileset);
 
-    if (mTilesets.value(tileset) == 0) {
-        mTilesets.remove(tileset);
-        if (!tileset->imageSource().isEmpty())
-            mWatcher->removePath(tileset->imageSource());
-    }
-}
-
-/**
- * Convenience method to add references to multiple tilesets.
- * @see addReference
- */
-void TilesetManager::addReferences(const QVector<SharedTileset> &tilesets)
-{
-    for (const SharedTileset &tileset : tilesets)
-        addReference(tileset);
-}
-
-/**
- * Convenience method to remove references from multiple tilesets.
- * @see removeReference
- */
-void TilesetManager::removeReferences(const QVector<SharedTileset> &tilesets)
-{
-    for (const SharedTileset &tileset : tilesets)
-        removeReference(tileset);
+    if (tileset->imageSource().isLocalFile())
+        mWatcher->removePath(tileset->imageSource().toLocalFile());
 }
 
 /**
  * Forces a tileset to reload.
  */
-void TilesetManager::reloadImages(const SharedTileset &tileset)
+void TilesetManager::reloadImages(Tileset *tileset)
 {
     if (!mTilesets.contains(tileset))
         return;
 
     if (tileset->isCollection()) {
-        for (Tile *tile : tileset->tiles())
-            tile->setImage(QPixmap(tile->imageSource()));
-        emit tilesetImagesChanged(tileset.data());
+        for (Tile *tile : tileset->tiles()) {
+            // todo: trigger reload of remote files
+            if (tile->imageSource().isLocalFile())
+                tile->setImage(QPixmap(tile->imageSource().toLocalFile()));
+        }
+        emit tilesetImagesChanged(tileset);
     } else {
         if (tileset->loadImage())
-            emit tilesetImagesChanged(tileset.data());
+            emit tilesetImagesChanged(tileset);
     }
 }
 
@@ -219,14 +191,15 @@ bool TilesetManager::animateTiles() const
 }
 
 void TilesetManager::tilesetImageSourceChanged(const Tileset &tileset,
-                                               const QString &oldImageSource)
+                                               const QUrl &oldImageSource)
 {
-    Q_ASSERT(mTilesets.contains(tileset.sharedPointer()));
-    Q_ASSERT(!oldImageSource.isEmpty());
-    Q_ASSERT(!tileset.imageSource().isEmpty());
+    Q_ASSERT(mTilesets.contains(const_cast<Tileset*>(&tileset)));
 
-    mWatcher->removePath(oldImageSource);
-    mWatcher->addPath(tileset.imageSource());
+    if (oldImageSource.isLocalFile())
+        mWatcher->removePath(oldImageSource.toLocalFile());
+
+    if (tileset.imageSource().isLocalFile())
+        mWatcher->addPath(tileset.imageSource().toLocalFile());
 }
 
 void TilesetManager::fileChanged(const QString &path)
@@ -245,15 +218,11 @@ void TilesetManager::fileChanged(const QString &path)
 
 void TilesetManager::fileChangedTimeout()
 {
-    QMapIterator<SharedTileset, int> it(mTilesets);
-
-    while (it.hasNext()) {
-        const SharedTileset &tileset = it.next().key();
-
-        QString fileName = tileset->imageSource();
+    for (Tileset *tileset : qAsConst(mTilesets)) {
+        const QString fileName = tileset->imageSource().toLocalFile();
         if (mChangedFiles.contains(fileName))
             if (tileset->loadFromImage(fileName))
-                emit tilesetImagesChanged(tileset.data());
+                emit tilesetImagesChanged(tileset);
     }
 
     mChangedFiles.clear();
@@ -268,17 +237,14 @@ void TilesetManager::resetTileAnimations()
     // TODO: This could be more optimal by keeping track of the list of
     // actually animated tiles
 
-    QMapIterator<SharedTileset, int> it(mTilesets);
-
-    while (it.hasNext()) {
-        const SharedTileset &tileset = it.next().key();
+    for (Tileset *tileset : qAsConst(mTilesets)) {
         bool imageChanged = false;
 
         for (Tile *tile : tileset->tiles())
             imageChanged |= tile->resetAnimation();
 
         if (imageChanged)
-            emit repaintTileset(tileset.data());
+            emit repaintTileset(tileset);
     }
 }
 
@@ -287,17 +253,14 @@ void TilesetManager::advanceTileAnimations(int ms)
     // TODO: This could be more optimal by keeping track of the list of
     // actually animated tiles
 
-    QMapIterator<SharedTileset, int> it(mTilesets);
-
-    while (it.hasNext()) {
-        const SharedTileset &tileset = it.next().key();
+    for (Tileset *tileset : qAsConst(mTilesets)) {
         bool imageChanged = false;
 
         for (Tile *tile : tileset->tiles())
             imageChanged |= tile->advanceAnimation(ms);
 
         if (imageChanged)
-            emit repaintTileset(tileset.data());
+            emit repaintTileset(tileset);
     }
 }
 
