@@ -35,6 +35,8 @@ AbstractTool::AbstractTool(const QString &name, const QIcon &icon,
     , mIcon(icon)
     , mShortcut(shortcut)
     , mEnabled(false)
+    , mTile(nullptr)
+    , mObjectTemplate(nullptr)
     , mMapDocument(nullptr)
 {
 }
@@ -75,6 +77,11 @@ void AbstractTool::keyPressed(QKeyEvent *event)
     event->ignore();
 }
 
+void AbstractTool::mouseDoubleClicked(QGraphicsSceneMouseEvent *event)
+{
+    mousePressed(event);
+}
+
 void AbstractTool::setMapDocument(MapDocument *mapDocument)
 {
     if (mMapDocument == mapDocument)
