@@ -52,10 +52,26 @@ void AutomappingManager::autoMap()
         return;
 
     Map *map = mMapDocument->map();
-    int w = map->width();
-    int h = map->height();
+    QRegion region = mMapDocument->selectedArea();
 
-    autoMapInternal(QRect(0, 0, w, h), nullptr);
+    if (region.isEmpty()) {
+        if (map->infinite()) {
+            LayerIterator iterator(map);
+
+            QRect bounds;
+            while (Layer *layer = iterator.next()) {
+                if (TileLayer *tileLayer = dynamic_cast<TileLayer*>(layer))
+                    bounds = bounds.united(tileLayer->bounds());
+            }
+            region = bounds;
+        } else {
+            int w = map->width();
+            int h = map->height();
+            region = QRect(0, 0, w, h);
+        }
+    }
+
+    autoMapInternal(region, nullptr);
 }
 
 void AutomappingManager::autoMap(const QRegion &where, Layer *touchedLayer)
@@ -167,7 +183,7 @@ bool AutomappingManager::loadFile(const QString &filePath)
             AutoMapper *autoMapper = new AutoMapper(mMapDocument, rules.take(), rulePath);
 
             mWarning += autoMapper->warningString();
-            const QString error = autoMapper->errorString(); 
+            const QString error = autoMapper->errorString();
             if (error.isEmpty()) {
                 mAutoMappers.append(autoMapper);
             } else {
