@@ -133,13 +133,22 @@ void BrushItem::paint(QPainter *painter,
     QColor insideMapHighlight = QApplication::palette().highlight().color();
     insideMapHighlight.setAlpha(64);
     QColor outsideMapHighlight = QColor(255, 0, 0, 64);
-    
+
     int mapWidth = mMapDocument->map()->width();
     int mapHeight = mMapDocument->map()->height();
     QRegion mapRegion = QRegion(0, 0, mapWidth, mapHeight);
+
+    if (!mMapDocument->currentLayer()->isUnlocked())
+        mapRegion = QRegion();
+
     QRegion insideMapRegion = mRegion.intersected(mapRegion);
     QRegion outsideMapRegion = mRegion.subtracted(mapRegion);
 
+    if (mMapDocument->map()->infinite()) {
+        insideMapRegion = mRegion;
+        outsideMapRegion = QRegion();
+    }
+
     const MapRenderer *renderer = mMapDocument->renderer();
     if (mTileLayer) {
         const qreal opacity = painter->opacity();
