@@ -27,9 +27,10 @@
 #include "brushitem.h"
 #include "tilepainter.h"
 #include "tile.h"
-#include "mapscene.h"
 #include "mapdocument.h"
 #include "painttilelayer.h"
+#include "staggeredrenderer.h"
+#include "stampactions.h"
 
 #include <QApplication>
 
@@ -37,15 +38,12 @@ using namespace Tiled;
 using namespace Tiled::Internal;
 
 BucketFillTool::BucketFillTool(QObject *parent)
-    : AbstractTileTool(tr("Bucket Fill Tool"),
-                       QIcon(QLatin1String(
-                               ":images/22x22/stock-tool-bucket-fill.png")),
-                       QKeySequence(tr("F")),
-                       parent)
-    , mIsActive(false)
-    , mLastShiftStatus(false)
-    , mIsRandom(false)
-    , mLastRandomStatus(false)
+    : AbstractTileFillTool(tr("Bucket Fill Tool"),
+                           QIcon(QLatin1String(
+                                   ":images/22x22/stock-tool-bucket-fill.png")),
+                           QKeySequence(tr("F")),
+                           nullptr,
+                           parent)
 {
 }
 
@@ -53,45 +51,15 @@ BucketFillTool::~BucketFillTool()
 {
 }
 
-void BucketFillTool::activate(MapScene *scene)
-{
-    AbstractTileTool::activate(scene);
-
-    mIsActive = true;
-    tilePositionChanged(tilePosition());
-}
-
-void BucketFillTool::deactivate(MapScene *scene)
-{
-    AbstractTileTool::deactivate(scene);
-
-    mFillRegion = QRegion();
-    mIsActive = false;
-}
-
-static void fillWithStamp(TileLayer &layer,
-                          const TileStamp &stamp,
-                          const QRegion &mask)
-{
-    const QSize size = stamp.maxSize();
-
-    // Fill the entire layer with random variations of the stamp
-    for (int y = 0; y < layer.height(); y += size.height()) {
-        for (int x = 0; x < layer.width(); x += size.width()) {
-            const TileStampVariation variation = stamp.randomVariation();
-            layer.setCells(x, y, variation.tileLayer());
-        }
-    }
-
-    // Erase tiles outside of the masked region. This can easily be faster than
-    // avoiding to place tiles outside of the region in the first place.
-    layer.erase(QRegion(0, 0, layer.width(), layer.height()) - mask);
-}
-
 void BucketFillTool::tilePositionChanged(const QPoint &tilePos)
 {
-    // Skip filling if the stamp is empty
-    if (mStamp.isEmpty())
+    AbstractTileFillTool::tilePositionChanged(tilePos);
+
+    if (isCapturing())
+        return;
+
+    // Skip filling if the stamp is empty and not in wangFill mode
+    if (mStamp.isEmpty() && mFillMethod != WangFill)
         return;
 
     // Make sure that a tile layer is selected
@@ -104,15 +72,6 @@ void BucketFillTool::tilePositionChanged(const QPoint &tilePos)
 
     TilePainter regionComputer(mapDocument(), tileLayer);
 
-    // If the stamp is a single tile, ignore it when making the region
-    if (!shiftPressed && mStamp.variations().size() == 1) {
-        const TileStampVariation &variation = mStamp.variations().first();
-        TileLayer *stampLayer = variation.tileLayer();
-        if (stampLayer->size() == QSize(1, 1) &&
-                stampLayer->cellAt(0, 0) == regionComputer.cellAt(tilePos))
-            return;
-    }
-
     // This clears the connections so we don't get callbacks
     clearConnections(mapDocument());
 
@@ -122,22 +81,34 @@ void BucketFillTool::tilePositionChanged(const QPoint &tilePos)
     if (!mFillRegion.contains(tilePos) || shiftPressed != mLastShiftStatus) {
 
         // Clear overlay to make way for a new one
-        clearOverlay();
+        AbstractTileFillTool::clearOverlay();
 
         // Cache information about how the fill region was created
         mLastShiftStatus = shiftPressed;
 
         // Get the new fill region
         if (!shiftPressed) {
-            // If not holding shift, a region is generated from the current pos
-            mFillRegion = regionComputer.computePaintableFillRegion(tilePos);
+            // If not holding shift, a region is computed from the current pos
+            bool computeRegion = true;
+
+            // If the stamp is a single tile, ignore that tile when making the region
+            if (mFillMethod != WangFill && mStamp.variations().size() == 1) {
+                const TileStampVariation &variation = mStamp.variations().first();
+                TileLayer *stampLayer = variation.tileLayer();
+                if (stampLayer->size() == QSize(1, 1) &&
+                        stampLayer->cellAt(0, 0) == regionComputer.cellAt(tilePos))
+                    computeRegion = false;
+            }
+
+            if (computeRegion)
+                mFillRegion = regionComputer.computePaintableFillRegion(tilePos);
         } else {
             // If holding shift, the region is the selection bounds
             mFillRegion = mapDocument()->selectedArea();
 
             // Fill region is the whole map if there is no selection
             if (mFillRegion.isEmpty())
-                mFillRegion = tileLayer->bounds();
+                mFillRegion = tileLayer->rect();
 
             // The mouse needs to be in the region
             if (!mFillRegion.contains(tilePos))
@@ -150,31 +121,38 @@ void BucketFillTool::tilePositionChanged(const QPoint &tilePos)
     if (mFillRegion.isEmpty())
         return;
 
-    if (mLastRandomStatus != mIsRandom) {
-        mLastRandomStatus = mIsRandom;
+    if (mLastFillMethod != mFillMethod) {
+        mLastFillMethod = mFillMethod;
         fillRegionChanged = true;
     }
 
     if (!mFillOverlay) {
         // Create a new overlay region
         const QRect fillBounds = mFillRegion.boundingRect();
-        mFillOverlay = SharedTileLayer(new TileLayer(QString(),
-                                                     fillBounds.x(),
-                                                     fillBounds.y(),
-                                                     fillBounds.width(),
-                                                     fillBounds.height()));
+        mFillOverlay = SharedTileLayer::create(QString(),
+                                               fillBounds.x(),
+                                               fillBounds.y(),
+                                               fillBounds.width(),
+                                               fillBounds.height());
     }
 
     // Paint the new overlay
-    if (!mIsRandom) {
+    switch (mFillMethod) {
+    case TileFill:
         if (fillRegionChanged || mStamp.variations().size() > 1) {
             fillWithStamp(*mFillOverlay, mStamp,
                           mFillRegion.translated(-mFillOverlay->position()));
             fillRegionChanged = true;
         }
-    } else {
+        break;
+    case RandomFill:
         randomFill(*mFillOverlay, mFillRegion);
         fillRegionChanged = true;
+        break;
+    case WangFill:
+        wangFill(*mFillOverlay, *tileLayer, mFillRegion);
+        fillRegionChanged = true;
+        break;
     }
 
     if (fillRegionChanged) {
@@ -187,11 +165,18 @@ void BucketFillTool::tilePositionChanged(const QPoint &tilePos)
 
 void BucketFillTool::mousePressed(QGraphicsSceneMouseEvent *event)
 {
+    AbstractTileFillTool::mousePressed(event);
+    if (event->isAccepted())
+        return;
+
     if (event->button() != Qt::LeftButton || mFillRegion.isEmpty())
         return;
     if (!brushItem()->isVisible())
         return;
 
+    if (!currentTileLayer()->isUnlocked())
+        return;
+
     const TileLayer *preview = mFillOverlay.data();
     if (!preview)
         return;
@@ -205,8 +190,10 @@ void BucketFillTool::mousePressed(QGraphicsSceneMouseEvent *event)
     paint->setText(QCoreApplication::translate("Undo Commands", "Fill Area"));
 
     if (!mMissingTilesets.isEmpty()) {
-        for (const SharedTileset &tileset : mMissingTilesets)
-            new AddTileset(mapDocument(), tileset, paint);
+        for (const SharedTileset &tileset : mMissingTilesets) {
+            if (!mapDocument()->map()->tilesets().contains(tileset))
+                new AddTileset(mapDocument(), tileset, paint);
+        }
 
         mMissingTilesets.clear();
     }
@@ -216,10 +203,6 @@ void BucketFillTool::mousePressed(QGraphicsSceneMouseEvent *event)
     emit mapDocument()->regionEdited(fillRegion, currentTileLayer());
 }
 
-void BucketFillTool::mouseReleased(QGraphicsSceneMouseEvent *)
-{
-}
-
 void BucketFillTool::modifiersChanged(Qt::KeyboardModifiers)
 {
     // Don't need to recalculate fill region if there was no fill region
@@ -233,32 +216,8 @@ void BucketFillTool::languageChanged()
 {
     setName(tr("Bucket Fill Tool"));
     setShortcut(QKeySequence(tr("F")));
-}
 
-void BucketFillTool::mapDocumentChanged(MapDocument *oldDocument,
-                                        MapDocument *newDocument)
-{
-    AbstractTileTool::mapDocumentChanged(oldDocument, newDocument);
-
-    clearConnections(oldDocument);
-
-    if (newDocument)
-        updateRandomListAndMissingTilesets();
-
-    clearOverlay();
-}
-
-void BucketFillTool::setStamp(const TileStamp &stamp)
-{
-    // Clear any overlay that we presently have with an old stamp
-    clearOverlay();
-
-    mStamp = stamp;
-
-    updateRandomListAndMissingTilesets();
-
-    if (mIsActive && brushItem()->isVisible())
-        tilePositionChanged(tilePosition());
+    mStampActions->languageChanged();
 }
 
 void BucketFillTool::clearOverlay()
@@ -267,9 +226,7 @@ void BucketFillTool::clearOverlay()
     // risk of getting a callback and causing an infinite loop
     clearConnections(mapDocument());
 
-    brushItem()->clear();
-    mFillOverlay.clear();
-    mFillRegion = QRegion();
+    AbstractTileFillTool::clearOverlay();
 }
 
 void BucketFillTool::makeConnections()
@@ -305,50 +262,3 @@ void BucketFillTool::clearConnections(MapDocument *mapDocument)
     disconnect(mapDocument, &MapDocument::selectedAreaChanged,
                this, &BucketFillTool::clearOverlay);
 }
-
-void BucketFillTool::setRandom(bool value)
-{
-    if (mIsRandom == value)
-        return;
-
-    mIsRandom = value;
-    updateRandomListAndMissingTilesets();
-
-    // Don't need to recalculate fill region if there was no fill region
-    if (!mFillOverlay)
-        return;
-
-    tilePositionChanged(tilePosition());
-}
-
-void BucketFillTool::randomFill(TileLayer &tileLayer, const QRegion &region) const
-{
-    if (region.isEmpty() || mRandomCellPicker.isEmpty())
-        return;
-
-    for (const QRect &rect : region.translated(-tileLayer.position()).rects()) {
-        for (int _x = rect.left(); _x <= rect.right(); ++_x) {
-            for (int _y = rect.top(); _y <= rect.bottom(); ++_y) {
-                tileLayer.setCell(_x, _y,
-                                  mRandomCellPicker.pick());
-            }
-        }
-    }
-}
-
-void BucketFillTool::updateRandomListAndMissingTilesets()
-{
-    mRandomCellPicker.clear();
-    mMissingTilesets.clear();
-
-    for (const TileStampVariation &variation : mStamp.variations()) {
-        mapDocument()->unifyTilesets(variation.map, mMissingTilesets);
-
-        if (mIsRandom) {
-            for (const Cell &cell : *variation.tileLayer()) {
-                if (const Tile *tile = cell.tile())
-                    mRandomCellPicker.add(cell, tile->probability());
-            }
-        }
-    }
-}
