@@ -49,11 +49,13 @@ CommandDataModel::CommandDataModel(QObject *parent)
         // warning when clicking the command button for the first time
         Command command(false);
 #ifdef Q_OS_LINUX
-        command.command = QLatin1String("gedit %mapfile");
+        command.executable = QLatin1String("gedit");
+        command.arguments = QLatin1String("%mapfile");
 #elif defined(Q_OS_MAC)
-        command.command = QLatin1String("open -t %mapfile");
+        command.executable = QLatin1String("open");
+        command.arguments = QLatin1String("-t %mapfile");
 #endif
-        if (!command.command.isEmpty()) {
+        if (!command.executable.isEmpty()) {
             command.name = tr("Open in text editor");
             mCommands.push_back(command);
         }
@@ -400,7 +402,7 @@ bool CommandDataModel::dropMimeData(const QMimeData *data, Qt::DropAction, int,
                 if (dstRow == mCommands.size()) {
                     append(Command(addr->isEnabled,
                                    tr("%1 (copy)").arg(addr->name),
-                                   addr->command));
+                                   addr->executable, addr->arguments));
                     return true;
                 }
             }
@@ -423,12 +425,28 @@ bool CommandDataModel::dropMimeData(const QMimeData *data, Qt::DropAction, int,
     return false;
 }
 
-void CommandDataModel::setCommand(const QModelIndex &index, const QString &value)
+void CommandDataModel::setExecutable(const QModelIndex &index, const QString &value)
 {
     const bool isNormalRow = index.row() < mCommands.size();
 
     if (isNormalRow)
-        mCommands[index.row()].command = value;
+        mCommands[index.row()].executable = value;
+}
+
+void CommandDataModel::setArguments(const QModelIndex &index, const QString &value)
+{
+    const bool isNormalRow = index.row() < mCommands.size();
+
+    if (isNormalRow)
+        mCommands[index.row()].arguments = value;
+}
+
+void CommandDataModel::setWorkingDirectory(const QModelIndex &index, const QString &value)
+{
+    const bool isNormalRow = index.row() < mCommands.size();
+
+    if (isNormalRow)
+        mCommands[index.row()].workingDirectory = value;
 }
 
 void CommandDataModel::setShortcut(const QModelIndex &index, const QKeySequence &value)
@@ -443,6 +461,14 @@ void CommandDataModel::setShortcut(const QModelIndex &index, const QKeySequence
     }
 }
 
+void CommandDataModel::setShowOutput(const QModelIndex &index, bool value)
+{
+    const bool isNormalRow = index.row() < mCommands.size();
+
+    if (isNormalRow)
+        mCommands[index.row()].showOutput = value;
+}
+
 void CommandDataModel::setSaveBeforeExecute(const QModelIndex &index, bool value)
 {
     const bool isNormalRow = index.row() < mCommands.size();
