@@ -49,17 +49,29 @@ CommandDialog::CommandDialog(QWidget *parent)
     connect(mUi->saveBox, &QCheckBox::stateChanged,
             this, &CommandDialog::setSaveBeforeExecute);
 
-    connect(mUi->keySequenceEdit, &QKeySequenceEdit::keySequenceChanged, 
+    connect(mUi->outputBox, &QCheckBox::stateChanged,
+            this, &CommandDialog::setShowOutput);
+
+    connect(mUi->keySequenceEdit, &QKeySequenceEdit::keySequenceChanged,
             this, &CommandDialog::setShortcut);
 
-    connect(mUi->commandEdit, &QLineEdit::textChanged,
-            this, &CommandDialog::setCommand);
+    connect(mUi->executableEdit, &QLineEdit::textChanged,
+            this, &CommandDialog::setExecutable);
 
-    connect(mUi->treeView->selectionModel(), &QItemSelectionModel::currentChanged, 
+    connect(mUi->argumentsEdit, &QLineEdit::textChanged,
+            this, &CommandDialog::setArguments);
+
+    connect(mUi->workingDirectoryEdit, &QLineEdit::textChanged,
+            this, &CommandDialog::setWorkingDirectory);
+
+    connect(mUi->treeView->selectionModel(), &QItemSelectionModel::currentChanged,
             this, &CommandDialog::updateWidgets);
 
-    connect(mUi->browseButton, &QPushButton::clicked,
-            this, &CommandDialog::openFileDialog);
+    connect(mUi->exBrowseButton, &QPushButton::clicked,
+            this, &CommandDialog::browseExecutable);
+
+    connect(mUi->wdBrowseButton, &QPushButton::clicked,
+            this, &CommandDialog::browseWorkingDirectory);
 }
 
 CommandDialog::~CommandDialog()
@@ -91,11 +103,33 @@ void CommandDialog::setSaveBeforeExecute(int state)
         mUi->treeView->model()->setSaveBeforeExecute(current, state);
 }
 
-void CommandDialog::setCommand(const QString &text)
+void CommandDialog::setShowOutput(int state)
 {
     const QModelIndex &current = mUi->treeView->currentIndex();
     if (current.row() < mUi->treeView->model()->rowCount())
-        mUi->treeView->model()->setCommand(current, text);
+        mUi->treeView->model()->setShowOutput(current, state);
+}
+
+
+void CommandDialog::setExecutable(const QString &text)
+{
+    const QModelIndex &current = mUi->treeView->currentIndex();
+    if (current.row() < mUi->treeView->model()->rowCount())
+        mUi->treeView->model()->setExecutable(current, text);
+}
+
+void CommandDialog::setArguments(const QString &text)
+{
+    const QModelIndex &current = mUi->treeView->currentIndex();
+    if (current.row() < mUi->treeView->model()->rowCount())
+        mUi->treeView->model()->setArguments(current, text);
+}
+
+void CommandDialog::setWorkingDirectory(const QString &text)
+{
+    const QModelIndex &current = mUi->treeView->currentIndex();
+    if (current.row() < mUi->treeView->model()->rowCount())
+        mUi->treeView->model()->setWorkingDirectory(current, text);
 }
 
 void CommandDialog::updateWidgets(const QModelIndex &current, const QModelIndex &)
@@ -103,31 +137,50 @@ void CommandDialog::updateWidgets(const QModelIndex &current, const QModelIndex
     bool enable = (current.row() < mUi->treeView->model()->rowCount() - 1);
 
     mUi->saveBox->setEnabled(enable);
-    mUi->commandEdit->setEnabled(enable);
-    mUi->browseButton->setEnabled(enable);
+    mUi->executableEdit->setEnabled(enable);
+    mUi->argumentsEdit->setEnabled(enable);
+    mUi->workingDirectoryEdit->setEnabled(enable);
+    mUi->exBrowseButton->setEnabled(enable);
+    mUi->wdBrowseButton->setEnabled(enable);
     mUi->keySequenceEdit->setEnabled(enable);
     mUi->clearButton->setEnabled(enable);
-    mUi->saveBox->setEnabled(enable);
+    mUi->outputBox->setEnabled(enable);
 
     if (enable) {
         const Command command = mUi->treeView->model()->command(current);
-        mUi->commandEdit->setText(command.command);
+        mUi->executableEdit->setText(command.executable);
+        mUi->argumentsEdit->setText(command.arguments);
+        mUi->workingDirectoryEdit->setText(command.workingDirectory);
         mUi->keySequenceEdit->setKeySequence(command.shortcut);
         mUi->saveBox->setChecked(command.saveBeforeExecute);
+        mUi->outputBox->setChecked(command.showOutput);
     } else {
-        mUi->commandEdit->clear();
+        mUi->executableEdit->clear();
+        mUi->argumentsEdit->clear();
+        mUi->workingDirectoryEdit->clear();
         mUi->keySequenceEdit->clear();
     }
 }
 
-void CommandDialog::openFileDialog()
+void CommandDialog::browseExecutable()
 {
     QString caption = tr("Select Executable");
     QString dir = QStandardPaths::writableLocation(QStandardPaths::ApplicationsLocation);
     QString executableName = QFileDialog::getOpenFileName(this, caption, dir);
 
     if (!executableName.isEmpty())
-        mUi->commandEdit->setText(executableName);
+        mUi->executableEdit->setText(executableName);
+}
+
+void CommandDialog::browseWorkingDirectory()
+{
+    QString caption = tr("Select Working Directory");
+    QString dir = QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
+    QString workingDirectoryName = QFileDialog::getExistingDirectory(this, caption, dir,
+                            QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
+
+    if (!workingDirectoryName.isEmpty())
+        mUi->workingDirectoryEdit->setText(workingDirectoryName);
 }
 
 CommandTreeView::CommandTreeView(QWidget *parent)
