@@ -43,7 +43,7 @@ class EditPolygonTool : public AbstractObjectTool
 
 public:
     explicit EditPolygonTool(QObject *parent = nullptr);
-    ~EditPolygonTool();
+    ~EditPolygonTool() override;
 
     void activate(MapScene *scene) override;
     void deactivate(MapScene *scene) override;
@@ -57,13 +57,18 @@ public:
 
     void languageChanged() override;
 
+    bool hasSelectedHandles() const { return !mSelectedHandles.isEmpty(); }
+
+public slots:
+    void deleteNodes();
+
 private slots:
     void updateHandles();
     void objectsRemoved(const QList<MapObject *> &objects);
 
-    void deleteNodes();
     void joinNodes();
     void splitSegments();
+    void deleteSegment();
 
 private:
     enum Mode {
@@ -72,6 +77,8 @@ private:
         Moving
     };
 
+    void updateHover(const QPointF &pos);
+
     void setSelectedHandles(const QSet<PointHandle*> &handles);
     void setSelectedHandle(PointHandle *handle)
     { setSelectedHandles(QSet<PointHandle*>() << handle); }
@@ -80,7 +87,7 @@ private:
 
     void startSelecting();
 
-    void startMoving();
+    void startMoving(const QPointF &pos, Qt::KeyboardModifiers modifiers);
     void updateMovingItems(const QPointF &pos,
                            Qt::KeyboardModifiers modifiers);
     void finishMoving(const QPointF &pos);
@@ -89,6 +96,7 @@ private:
 
     SelectionRectangle *mSelectionRectangle;
     bool mMousePressed;
+    PointHandle *mHoveredHandle;
     PointHandle *mClickedHandle;
     MapObjectItem *mClickedObjectItem;
     QVector<QPointF> mOldHandlePositions;
@@ -100,7 +108,7 @@ private:
     Qt::KeyboardModifiers mModifiers;
 
     /// The list of handles associated with each selected map object
-    QMap<MapObjectItem*, QList<PointHandle*> > mHandles;
+    QMap<MapObject*, QList<PointHandle*> > mHandles;
     QSet<PointHandle*> mSelectedHandles;
 };
 
