@@ -75,7 +75,7 @@ public:
                       int row, int column,
                       const QModelIndex &parent) override;
 
-    QModelIndex index(Layer *layer) const;
+    QModelIndex index(Layer *layer, int column = 0) const;
     Layer *toLayer(const QModelIndex &index) const;
 
     MapDocument *mapDocument() const;
@@ -86,7 +86,8 @@ public:
     void replaceLayer(Layer *layer, Layer *replacement);
 
     void setLayerVisible(Layer *layer, bool visible);
-    void setLayerOpacity(Layer *layer, float opacity);
+    void setLayerLocked(Layer *layer, bool locked);
+    void setLayerOpacity(Layer *layer, qreal opacity);
     void setLayerOffset(Layer *layer, const QPointF &offset);
 
     void renameLayer(Layer *layer, const QString &name);
