@@ -213,7 +213,8 @@ public:
     void unifyTilesets(Map *map);
     void unifyTilesets(Map *map, QVector<SharedTileset> &missingTilesets);
 
-    void emitEditLayerNameRequested();
+    bool allowHidingObjects() const { return mAllowHidingObjects; }
+    void setAllowHidingObjects(bool value) { mAllowHidingObjects = value; }
 
 signals:
     /**
@@ -330,8 +331,9 @@ private:
     QRegion mSelectedArea;
     QList<MapObject*> mSelectedObjects;
     MapRenderer *mRenderer;
-    Layer* mCurrentLayer;
+    Layer *mCurrentLayer;
     MapObjectModel *mMapObjectModel;
+    bool mAllowHidingObjects = true;
 };
 
 } // namespace Internal
