@@ -66,6 +66,7 @@ MapDocumentActionHandler::MapDocumentActionHandler(QObject *parent)
     mActionSelectNone->setShortcut(tr("Ctrl+Shift+A"));
 
     mActionCropToSelection = new QAction(this);
+    mActionAutocrop = new QAction(this);
 
     QIcon addTileLayerIcon(QLatin1String(":/images/16x16/layer-tile.png"));
     QIcon addObjectLayerIcon(QLatin1String(":/images/16x16/layer-object.png"));
@@ -144,6 +145,7 @@ MapDocumentActionHandler::MapDocumentActionHandler(QObject *parent)
     connect(mActionSelectInverse, &QAction::triggered, this, &MapDocumentActionHandler::selectInverse);
     connect(mActionSelectNone, &QAction::triggered, this, &MapDocumentActionHandler::selectNone);
     connect(mActionCropToSelection, &QAction::triggered, this, &MapDocumentActionHandler::cropToSelection);
+    connect(mActionAutocrop, &QAction::triggered, this, &MapDocumentActionHandler::autocrop);
     connect(mActionAddTileLayer, &QAction::triggered, this, &MapDocumentActionHandler::addTileLayer);
     connect(mActionAddObjectGroup, &QAction::triggered, this, &MapDocumentActionHandler::addObjectGroup);
     connect(mActionAddImageLayer, &QAction::triggered, this, &MapDocumentActionHandler::addImageLayer);
@@ -181,6 +183,7 @@ void MapDocumentActionHandler::retranslateUi()
     mActionSelectInverse->setText(tr("Invert S&election"));
     mActionSelectNone->setText(tr("Select &None"));
     mActionCropToSelection->setText(tr("&Crop to Selection"));
+    mActionAutocrop->setText(tr("Autocrop"));
 
     mActionAddTileLayer->setText(tr("&Tile Layer"));
     mActionAddObjectGroup->setText(tr("&Object Layer"));
@@ -224,6 +227,8 @@ void MapDocumentActionHandler::setMapDocument(MapDocument *mapDocument)
                 this, &MapDocumentActionHandler::updateActions);
         connect(mapDocument, &MapDocument::selectedObjectsChanged,
                 this, &MapDocumentActionHandler::updateActions);
+        connect(mapDocument, &MapDocument::mapChanged,
+                this, &MapDocumentActionHandler::updateActions);
     }
 
     emit mapDocumentChanged(mMapDocument);
@@ -266,7 +271,7 @@ void MapDocumentActionHandler::cut()
         return;
 
     Layer *currentLayer = mMapDocument->currentLayer();
-    if (!currentLayer)
+    if (!currentLayer || !currentLayer->isUnlocked())
         return;
 
     TileLayer *tileLayer = dynamic_cast<TileLayer*>(currentLayer);
@@ -302,7 +307,7 @@ void MapDocumentActionHandler::delete_()
         return;
 
     Layer *currentLayer = mMapDocument->currentLayer();
-    if (!currentLayer)
+    if (!currentLayer || !currentLayer->isUnlocked())
         return;
 
     TileLayer *tileLayer = dynamic_cast<TileLayer*>(currentLayer);
@@ -333,8 +338,9 @@ void MapDocumentActionHandler::selectAll()
         return;
 
     if (TileLayer *tileLayer = layer->asTileLayer()) {
-        QRect all(tileLayer->x(), tileLayer->y(),
-                  tileLayer->width(), tileLayer->height());
+        QRect all = tileLayer->rect();
+        if (mMapDocument->map()->infinite())
+            all = tileLayer->bounds();
 
         if (mMapDocument->selectedArea() == all)
             return;
@@ -342,7 +348,8 @@ void MapDocumentActionHandler::selectAll()
         QUndoCommand *command = new ChangeSelectedArea(mMapDocument, all);
         mMapDocument->undoStack()->push(command);
     } else if (ObjectGroup *objectGroup = layer->asObjectGroup()) {
-        mMapDocument->setSelectedObjects(objectGroup->objects());
+        if (objectGroup->isUnlocked())
+            mMapDocument->setSelectedObjects(objectGroup->objects());
     }
 }
 
@@ -356,7 +363,9 @@ void MapDocumentActionHandler::selectInverse()
         return;
 
     if (TileLayer *tileLayer = layer->asTileLayer()) {
-        QRegion all(tileLayer->bounds());
+        QRegion all = tileLayer->rect();
+        if (mMapDocument->map()->infinite())
+            all = tileLayer->bounds();
 
         QUndoCommand *command = new ChangeSelectedArea(mMapDocument, all - mMapDocument->selectedArea());
         mMapDocument->undoStack()->push(command);
@@ -388,7 +397,7 @@ void MapDocumentActionHandler::selectNone()
 
         QUndoCommand *command = new ChangeSelectedArea(mMapDocument, QRegion());
         mMapDocument->undoStack()->push(command);
-    } else if (layer->asObjectGroup()) {
+    } else if (layer->isObjectGroup()) {
         mMapDocument->setSelectedObjects(QList<MapObject*>());
     }
 }
@@ -425,6 +434,12 @@ void MapDocumentActionHandler::cropToSelection()
     mMapDocument->resizeMap(bounds.size(), -bounds.topLeft(), true);
 }
 
+void MapDocumentActionHandler::autocrop()
+{
+    if (mMapDocument)
+        mMapDocument->autocropMap();
+}
+
 void MapDocumentActionHandler::addTileLayer()
 {
     if (mMapDocument)
@@ -477,7 +492,7 @@ void MapDocumentActionHandler::layerVia(MapDocumentActionHandler::LayerViaVarian
         break;
     }
     case Layer::ObjectGroupType: {
-        selectedObjects = mMapDocument->selectedObjects();
+        selectedObjects = mMapDocument->selectedObjectsOrdered();
         if (selectedObjects.isEmpty())
             return;
 
@@ -625,6 +640,12 @@ void MapDocumentActionHandler::moveObjectsToGroup(ObjectGroup *objectGroup)
     }
 }
 
+void MapDocumentActionHandler::selectAllInstances(const ObjectTemplate *objectTemplate)
+{
+    if (mMapDocument)
+        mMapDocument->selectAllInstances(objectTemplate);
+}
+
 void MapDocumentActionHandler::updateActions()
 {
     Map *map = nullptr;
@@ -668,6 +689,8 @@ void MapDocumentActionHandler::updateActions()
 
     mActionCropToSelection->setEnabled(!selection.isEmpty());
 
+    mActionAutocrop->setEnabled(currentLayer && currentLayer->isTileLayer() && !map->infinite());
+
     mActionAddTileLayer->setEnabled(map);
     mActionAddObjectGroup->setEnabled(map);
     mActionAddImageLayer->setEnabled(map);
