@@ -22,6 +22,7 @@
 #pragma once
 
 #include <QObject>
+#include "mapobject.h"
 
 class QAction;
 class QMenu;
@@ -29,6 +30,7 @@ class QMenu;
 namespace Tiled {
 
 class ObjectGroup;
+class MapObject;
 
 namespace Internal {
 
@@ -49,7 +51,7 @@ class MapDocumentActionHandler : public QObject
 
 public:
     explicit MapDocumentActionHandler(QObject *parent = nullptr);
-    ~MapDocumentActionHandler();
+    ~MapDocumentActionHandler() override;
 
     static MapDocumentActionHandler *instance() { return mInstance; }
 
@@ -62,6 +64,7 @@ public:
     QAction *actionSelectInverse() const { return mActionSelectInverse; }
     QAction *actionSelectNone() const { return mActionSelectNone; }
     QAction *actionCropToSelection() const { return mActionCropToSelection; }
+    QAction *actionAutocrop() const { return mActionAutocrop; }
 
     QAction *actionAddTileLayer() const { return mActionAddTileLayer; }
     QAction *actionAddObjectGroup() const { return mActionAddObjectGroup; }
@@ -103,6 +106,7 @@ public slots:
     void copyPosition();
 
     void cropToSelection();
+    void autocrop();
 
     void addTileLayer();
     void addObjectGroup();
@@ -128,6 +132,8 @@ public slots:
     void removeObjects();
     void moveObjectsToGroup(ObjectGroup *);
 
+    void selectAllInstances(const ObjectTemplate *objectTemplate);
+
 private slots:
     void updateActions();
 
@@ -138,6 +144,7 @@ private:
     QAction *mActionSelectInverse;
     QAction *mActionSelectNone;
     QAction *mActionCropToSelection;
+    QAction *mActionAutocrop;
 
     QAction *mActionAddTileLayer;
     QAction *mActionAddObjectGroup;
