@@ -25,6 +25,7 @@
 #include "map.h"
 #include "mapdocument.h"
 #include "maprenderer.h"
+#include "mapscene.h"
 #include "mapview.h"
 #include "utils.h"
 #include "zoomable.h"
@@ -43,10 +44,11 @@ MiniMap::MiniMap(QWidget *parent)
     , mDragging(false)
     , mMouseMoveCursorState(false)
     , mRedrawMapImage(false)
-    , mRenderFlags(MiniMapRenderer::DrawTiles
-                   | MiniMapRenderer::DrawObjects
-                   | MiniMapRenderer::DrawImages
-                   | MiniMapRenderer::IgnoreInvisibleLayer)
+    , mRenderFlags(MiniMapRenderer::DrawTileLayers
+                   | MiniMapRenderer::DrawMapObjects
+                   | MiniMapRenderer::DrawImageLayers
+                   | MiniMapRenderer::IgnoreInvisibleLayer
+                   | MiniMapRenderer::SmoothPixmapTransform)
 {
     setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
     setMinimumSize(50, 50);
@@ -173,7 +175,7 @@ void MiniMap::renderMapToImage()
 #else
     const QSize viewSize = contentsRect().size() * devicePixelRatio();
 #endif
-    QSize mapSize = renderer->mapSize();
+    QSize mapSize = renderer->mapBoundingRect().size();
 
     if (mapSize.isEmpty()) {
         mMapImage = QImage();
@@ -194,7 +196,7 @@ void MiniMap::renderMapToImage()
     if (imageSize.isEmpty())
         return;
 
-    MiniMapRenderer miniMapRenderer(mMapDocument);
+    MiniMapRenderer miniMapRenderer(mMapDocument->map());
     miniMapRenderer.renderToImage(mMapImage, mRenderFlags);
 }
 
@@ -217,7 +219,7 @@ void MiniMap::redrawTimeout()
 }
 
 void MiniMap::wheelEvent(QWheelEvent *event)
-{    
+{
     if (event->orientation() == Qt::Vertical) {
         centerViewOnLocalPixel(event->pos(), event->delta());
         return;
@@ -227,7 +229,7 @@ void MiniMap::wheelEvent(QWheelEvent *event)
 }
 
 void MiniMap::mousePressEvent(QMouseEvent *event)
-{       
+{
     if (event->button() == Qt::LeftButton) {
         QPoint cursorPos = event->pos();
         QRect viewPort = viewportRect();
@@ -270,7 +272,7 @@ void MiniMap::mouseReleaseEvent(QMouseEvent *event)
 }
 
 void MiniMap::mouseMoveEvent(QMouseEvent *event)
-{    
+{
     if (mDragging) {
         centerViewOnLocalPixel(event->pos() + mDragOffset);
         return;
@@ -297,7 +299,7 @@ QRect MiniMap::viewportRect() const
     if (!mapView)
         return QRect(0, 0, 1, 1);
 
-    const QRectF sceneRect = mapView->sceneRect();
+    const QRectF sceneRect = mapView->mapScene()->sceneRect();
     const QRectF viewRect = mapView->mapToScene(mapView->viewport()->geometry()).boundingRect();
     return QRect((viewRect.x() - sceneRect.x()) / sceneRect.width() * mImageRect.width() + mImageRect.x(),
                  (viewRect.y() - sceneRect.y()) / sceneRect.height() * mImageRect.height() + mImageRect.y(),
@@ -314,7 +316,7 @@ QPointF MiniMap::mapToScene(QPoint p) const
     if (!mapView)
         return QPointF();
 
-    const QRectF sceneRect = mapView->sceneRect();
+    const QRectF sceneRect = mapView->mapScene()->sceneRect();
     p -= mImageRect.topLeft();
     return QPointF(p.x() * (sceneRect.width() / mImageRect.width()) + sceneRect.x(),
                    p.y() * (sceneRect.height() / mImageRect.height()) + sceneRect.y());
