@@ -34,6 +34,8 @@
 #include <QGuiApplication>
 #include <QTimerEvent>
 
+#include "qtcompat_p.h"
+
 #include <cmath>
 
 namespace Tiled {
@@ -49,14 +51,14 @@ static QPointF alignmentOffset(QRectF &r, Alignment alignment)
 {
     switch (alignment) {
     case TopLeft:       break;
-    case Top:           return QPointF(r.width() / 2, 0);               break;
-    case TopRight:      return QPointF(r.width(), 0);                   break;
-    case Left:          return QPointF(0, r.height() / 2);              break;
-    case Center:        return QPointF(r.width() / 2, r.height() / 2);  break;
-    case Right:         return QPointF(r.width(), r.height() / 2);      break;
-    case BottomLeft:    return QPointF(0, r.height());                  break;
-    case Bottom:        return QPointF(r.width() / 2, r.height());      break;
-    case BottomRight:   return QPointF(r.width(), r.height());          break;
+    case Top:           return QPointF(r.width() / 2, 0);
+    case TopRight:      return QPointF(r.width(), 0);
+    case Left:          return QPointF(0, r.height() / 2);
+    case Center:        return QPointF(r.width() / 2, r.height() / 2);
+    case Right:         return QPointF(r.width(), r.height() / 2);
+    case BottomLeft:    return QPointF(0, r.height());
+    case Bottom:        return QPointF(r.width() / 2, r.height());
+    case BottomRight:   return QPointF(r.width(), r.height());
     }
     return QPointF();
 }
@@ -108,6 +110,8 @@ static QRectF objectBounds(const MapObject *object,
             QPolygonF screenPolygon = renderer->pixelToScreenCoords(bounds);
             return screenPolygon.boundingRect();
         }
+        case MapObject::Point:
+            return renderer->shape(object).boundingRect();
         case MapObject::Polygon:
         case MapObject::Polyline: {
             // Alignment is irrelevant for polygon objects since they have no size
@@ -203,7 +207,6 @@ void MapObjectOutline::paint(QPainter *painter,
 #else
     const int devicePixelRatio = painter->device()->devicePixelRatio();
 #endif
-
     const qreal dashLength = std::ceil(Utils::dpiScaled(3) * devicePixelRatio);
 
     // Draw a black dashed line above the white line
@@ -325,8 +328,10 @@ void MapObjectLabel::paint(QPainter *painter,
 }
 
 
-ObjectSelectionItem::ObjectSelectionItem(MapDocument *mapDocument)
-    : mMapDocument(mapDocument)
+ObjectSelectionItem::ObjectSelectionItem(MapDocument *mapDocument,
+                                         QGraphicsItem *parent)
+    : QGraphicsObject(parent)
+    , mMapDocument(mapDocument)
 {
     setFlag(QGraphicsItem::ItemHasNoContents);
 
@@ -541,11 +546,12 @@ void ObjectSelectionItem::addRemoveObjectLabels()
         }
     }
         // We want labels on selected objects regardless layer visibility
-        /*FALLTHROUGH*/
+        Q_FALLTHROUGH();
 
     case Preferences::SelectedObjectLabels:
         for (MapObject *object : mMapDocument->selectedObjects())
             ensureLabel(object);
+        break;
 
     case Preferences::NoObjectLabels:
         break;
