@@ -46,9 +46,9 @@
 #include <QGraphicsItem>
 #include <QGraphicsView>
 #include <QKeyEvent>
+#include <QMenu>
 #include <QTransform>
 #include <QUndoStack>
-#include <QMenu>
 
 #include "qtcompat_p.h"
 
@@ -693,6 +693,8 @@ void ObjectSelectionTool::modifiersChanged(Qt::KeyboardModifiers modifiers)
 
 void ObjectSelectionTool::languageChanged()
 {
+    AbstractObjectTool::languageChanged();
+
     setName(tr("Select Objects"));
     setShortcut(QKeySequence(tr("S")));
 }
@@ -976,6 +978,11 @@ void ObjectSelectionTool::updateHandleVisibility()
 
 void ObjectSelectionTool::objectsRemoved(const QList<MapObject *> &objects)
 {
+    if (mClickedObjectItem && objects.contains(mClickedObjectItem->mapObject()))
+        mClickedObjectItem = nullptr;
+    if (mHoveredObjectItem && objects.contains(mHoveredObjectItem->mapObject()))
+        mHoveredObjectItem = nullptr;
+
     if (mAction != Moving && mAction != Rotating && mAction != Resizing)
         return;
 
