@@ -37,10 +37,17 @@ OffsetMapDialog::OffsetMapDialog(MapDocument *mapDocument, QWidget *parent)
     mUi->setupUi(this);
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
-    if (mMapDocument->selectedArea().isEmpty())
-        disableBoundsSelectionCurrentArea();
-    else
-        mUi->boundsSelection->setCurrentIndex(1);
+    if (mMapDocument->selectedArea().isEmpty()) {
+        setBoundsSelection(WholeMap);
+        mUi->boundsSelection->setEnabled(false);
+    } else {
+        setBoundsSelection(CurrentSelectionArea);
+    }
+
+    boundsSelectionChanged();   // updates wrap checkboxes
+
+    connect(mUi->boundsSelection, SIGNAL(currentIndexChanged(int)),
+            this, SLOT(boundsSelectionChanged()));
 }
 
 OffsetMapDialog::~OffsetMapDialog()
@@ -73,13 +80,20 @@ QList<Layer *> OffsetMapDialog::affectedLayers() const
     return layers;
 }
 
+/**
+ * Returns the bounding rect that is to be affected by the offset operation.
+ *
+ * For infinite maps, when not using the currently selected area, the returned
+ * rect is empty.
+ */
 QRect OffsetMapDialog::affectedBoundingRect() const
 {
     QRect boundingRect;
 
     switch (boundsSelection()) {
     case WholeMap:
-        boundingRect = QRect(QPoint(0, 0), mMapDocument->map()->size());
+        if (!mMapDocument->map()->infinite())
+            boundingRect = QRect(QPoint(0, 0), mMapDocument->map()->size());
         break;
     case CurrentSelectionArea: {
         const QRegion &selection = mMapDocument->selectedArea();
@@ -115,6 +129,18 @@ OffsetMapDialog::BoundsSelection OffsetMapDialog::boundsSelection() const
     return CurrentSelectionArea;
 }
 
+void OffsetMapDialog::setBoundsSelection(BoundsSelection boundsSelection)
+{
+    switch (boundsSelection) {
+    case WholeMap:
+        mUi->boundsSelection->setCurrentIndex(0);
+        break;
+    case CurrentSelectionArea:
+        mUi->boundsSelection->setCurrentIndex(1);
+        break;
+    }
+}
+
 QPoint OffsetMapDialog::offset() const
 {
     return QPoint(mUi->xOffset->value(), mUi->yOffset->value());
@@ -130,10 +156,20 @@ bool OffsetMapDialog::wrapY() const
     return mUi->wrapY->isChecked();
 }
 
-void OffsetMapDialog::disableBoundsSelectionCurrentArea()
+void OffsetMapDialog::boundsSelectionChanged()
 {
-    mUi->boundsSelection->setEnabled(false);
-    mUi->boundsSelection->setCurrentIndex(0);
+    bool wrapEnabled = true;
+
+    if (boundsSelection() == WholeMap && mMapDocument->map()->infinite())
+        wrapEnabled = false;
+
+    mUi->wrapX->setEnabled(wrapEnabled);
+    mUi->wrapY->setEnabled(wrapEnabled);
+
+    if (!wrapEnabled) {
+        mUi->wrapX->setChecked(false);
+        mUi->wrapY->setChecked(false);
+    }
 }
 
 } // namespace Internal
