@@ -83,6 +83,8 @@ public:
      */
     void editCustomProperty(const QString &name);
 
+    QSize sizeHint() const override;
+
 protected:
     bool event(QEvent *event) override;
 
@@ -97,6 +99,7 @@ private slots:
     void tileChanged(Tile *tile);
     void tileTypeChanged(Tile *tile);
     void terrainChanged(Tileset *tileset, int index);
+    void wangSetChanged(Tileset *tileset, int index);
 
     void propertyAdded(Object *object, const QString &name);
     void propertyRemoved(Object *object, const QString &name);
@@ -121,6 +124,7 @@ private:
         HeightProperty,
         RotationProperty,
         VisibleProperty,
+        LockedProperty,
         OpacityProperty,
         TextProperty,
         TextAlignmentProperty,
@@ -151,7 +155,12 @@ private:
         TileProbabilityProperty,
         ColumnCountProperty,
         IdProperty,
-        CustomProperty
+        EdgeCountProperty,
+        CornerCountProperty,
+        WangColorProbabilityProperty,
+        CustomProperty,
+        InfiniteProperty,
+        TemplateProperty
     };
 
     void addMapProperties();
@@ -164,6 +173,8 @@ private:
     void addTilesetProperties();
     void addTileProperties();
     void addTerrainProperties();
+    void addWangSetProperties();
+    void addWangColorProperties();
 
     void applyMapValue(PropertyId id, const QVariant &val);
     void applyMapObjectValue(PropertyId id, const QVariant &val);
@@ -176,6 +187,8 @@ private:
     void applyTilesetValue(PropertyId id, const QVariant &val);
     void applyTileValue(PropertyId id, const QVariant &val);
     void applyTerrainValue(PropertyId id, const QVariant &val);
+    void applyWangSetValue(PropertyId id, const QVariant &val);
+    void applyWangColorValue(PropertyId id, const QVariant &val);
 
     QtVariantProperty *createProperty(PropertyId id,
                                       int type,
@@ -195,11 +208,12 @@ private:
     void removeProperties();
     void updateProperties();
     void updateCustomProperties();
-    void retranslateUi();
-    bool mUpdating;
-
     void updateCustomPropertyColor(const QString &name);
 
+    void retranslateUi();
+
+    bool mUpdating;
+    int mMapObjectFlags;
     Object *mObject;
     Document *mDocument;
     MapDocument *mMapDocument;
@@ -230,11 +244,5 @@ inline Object *PropertyBrowser::object() const
     return mObject;
 }
 
-inline void PropertyBrowser::retranslateUi()
-{
-    removeProperties();
-    addProperties();
-}
-
 } // namespace Internal
 } // namespace Tiled
