@@ -193,32 +193,24 @@ void TilePainter::erase(const QRegion &region)
     emit mMapDocument->regionChanged(paintable, mTileLayer);
 }
 
-static QRegion fillRegion(const TileLayer *layer, QPoint fillOrigin,
+static QRegion fillRegion(const TileLayer *layer,
+                          const QRegion &region,
+                          QPoint fillOrigin,
                           Map::Orientation orientation,
                           Map::StaggerAxis staggerAxis,
                           Map::StaggerIndex staggerIndex)
 {
-    // Create that region that will hold the fill
-    QRegion fillRegion;
-    QRect bounds = layer->map()->infinite() ? layer->bounds() : layer->rect();
-
-    // Silently quit if parameters are unsatisfactory
-    if (!bounds.contains(fillOrigin))
-        return fillRegion;
-
-    bounds.translate(-layer->position());
+    // Return empty region when the bounds do not contain the fill origin
+    if (!region.contains(fillOrigin))
+        return QRegion();
 
     // Cache cell that we will match other cells against
     const Cell matchCell = layer->cellAt(fillOrigin);
 
-    int startX = bounds.left();
-    int startY = bounds.top();
-    int endX = bounds.right() + 1;
-    int endY = bounds.bottom() + 1;
-    int layerWidth = endX - startX;
-    int layerHeight = endY - startY;
-
-    int indexOffset = -(startX + startY * layerWidth);
+    const QRect bounds = region.boundingRect();
+    const int width = bounds.width();
+    const int height = bounds.height();
+    const int indexOffset = -(bounds.left() + bounds.top() * width);
 
     const bool isStaggered = orientation == Map::Hexagonal || orientation == Map::Staggered;
 
@@ -228,25 +220,26 @@ static QRegion fillRegion(const TileLayer *layer, QPoint fillOrigin,
 
     // Create an array that will store which cells have been processed
     // This is faster than checking if a given cell is in the region/list
-    QVector<bool> processedCellsVec(layerWidth * layerHeight);
+    QVector<bool> processedCellsVec(width * height);
     bool *processedCells = processedCellsVec.data();
+    QRegion fillRegion;
 
     // Loop through queued positions and fill them, while at the same time
     // checking adjacent positions to see if they should be added
     while (!fillPositions.isEmpty()) {
         const QPoint currentPoint = fillPositions.dequeue();
-        const int startOfLine = currentPoint.y() * layerWidth;
+        const int startOfLine = currentPoint.y() * width;
 
         // Seek as far left as we can
         int left = currentPoint.x();
-        while (left > startX && layer->cellAt(left - 1, currentPoint.y()) == matchCell) {
+        while (left > bounds.left() && layer->cellAt(left - 1, currentPoint.y()) == matchCell) {
             --left;
             processedCells[indexOffset + startOfLine + left] = true;
         }
 
         // Seek as far right as we can
         int right = currentPoint.x();
-        while (right + 1 < endX && layer->cellAt(right + 1, currentPoint.y()) == matchCell) {
+        while (right < bounds.right() && layer->cellAt(right + 1, currentPoint.y()) == matchCell) {
             ++right;
             processedCells[indexOffset + startOfLine + right] = true;
         }
@@ -262,9 +255,9 @@ static QRegion fillRegion(const TileLayer *layer, QPoint fillOrigin,
             if (staggerAxis == Map::StaggerY) {
                 bool rowIsStaggered = ((layer->y() + currentPoint.y()) & 1) ^ staggerIndex;
                 if (rowIsStaggered)
-                    right = qMin(right + 1, endX - 1);
+                    right = qMin(right + 1, bounds.right());
                 else
-                    left = qMax(left - 1, startX);
+                    left = qMax(left - 1, bounds.left());
             } else {
                 leftColumnIsStaggered = ((layer->x() + left) & 1) ^ staggerIndex;
                 rightColumnIsStaggered = ((layer->x() + right) & 1) ^ staggerIndex;
@@ -277,7 +270,7 @@ static QRegion fillRegion(const TileLayer *layer, QPoint fillOrigin,
             bool adjacentCellAdded = false;
 
             for (int x = left; x <= right; ++x) {
-                const int index = y * layerWidth + x;
+                const int index = y * width + x;
 
                 if (!processedCells[indexOffset + index] && layer->cellAt(x, y) == matchCell) {
                     // Do not add the cell to the queue if an adjacent cell was added.
@@ -293,29 +286,29 @@ static QRegion fillRegion(const TileLayer *layer, QPoint fillOrigin,
             }
         };
 
-        if (currentPoint.y() > startY) {
+        if (currentPoint.y() > bounds.top()) {
             int _left = left;
             int _right = right;
 
             if (isStaggered && staggerAxis == Map::StaggerX) {
                 if (!leftColumnIsStaggered)
-                    _left = qMax(left - 1, startX);
+                    _left = qMax(left - 1, bounds.left());
                 if (!rightColumnIsStaggered)
-                    _right = qMin(right + 1, endX - 1);
+                    _right = qMin(right + 1, bounds.right());
             }
 
             findFillPositions(_left, _right, currentPoint.y() - 1);
         }
 
-        if (currentPoint.y() + 1 < endY) {
+        if (currentPoint.y() < bounds.bottom()) {
             int _left = left;
             int _right = right;
 
             if (isStaggered && staggerAxis == Map::StaggerX) {
                 if (leftColumnIsStaggered)
-                    _left = qMax(left - 1, startX);
+                    _left = qMax(left - 1, bounds.left());
                 if (rightColumnIsStaggered)
-                    _right = qMin(right + 1, endX - 1);
+                    _right = qMin(right + 1, bounds.right());
             }
 
             findFillPositions(_left, _right, currentPoint.y() + 1);
@@ -327,9 +320,23 @@ static QRegion fillRegion(const TileLayer *layer, QPoint fillOrigin,
 
 QRegion TilePainter::computePaintableFillRegion(const QPoint &fillOrigin) const
 {
-    QRegion region = computeFillRegion(fillOrigin);
-
+    const Map *map = mMapDocument->map();
     const QRegion &selection = mMapDocument->selectedArea();
+
+    QRegion bounds;
+
+    if (map->infinite())
+        bounds = selection.isEmpty() ? mTileLayer->bounds() : selection;
+    else
+        bounds = mTileLayer->rect();
+
+    QRegion region = fillRegion(mTileLayer,
+                                bounds.translated(-mTileLayer->position()),
+                                fillOrigin - mTileLayer->position(),
+                                map->orientation(), map->staggerAxis(), map->staggerIndex());
+
+    region.translate(mTileLayer->position());
+
     if (!selection.isEmpty())
         region &= selection;
 
@@ -338,9 +345,13 @@ QRegion TilePainter::computePaintableFillRegion(const QPoint &fillOrigin) const
 
 QRegion TilePainter::computeFillRegion(const QPoint &fillOrigin) const
 {
-    Map *map = mMapDocument->map();
-    QRegion region = fillRegion(mTileLayer, fillOrigin - mTileLayer->position(),
+    const Map *map = mMapDocument->map();
+    QRegion bounds = map->infinite() ? mTileLayer->bounds() : mTileLayer->rect();
+    QRegion region = fillRegion(mTileLayer,
+                                bounds.translated(-mTileLayer->position()),
+                                fillOrigin - mTileLayer->position(),
                                 map->orientation(), map->staggerAxis(), map->staggerIndex());
+
     return region.translated(mTileLayer->position());
 }
 
