@@ -42,7 +42,7 @@ struct TileStampVariation
     TileStampVariation(Map *map, qreal probability = 1.0)
         : map(map), probability(probability)
     {
-        Q_ASSERT(map->layerCount() == 1);
+        Q_ASSERT(map->layerCount() >= 1);
         Q_ASSERT(map->layerAt(0)->isTileLayer());
     }
 
@@ -83,7 +83,6 @@ public:
     void addVariation(Map *map, qreal probability = 1.0);
     void addVariation(const TileStampVariation &variation);
     Map *takeVariation(int index);
-    void deleteVariation(int index);
     bool isEmpty() const;
 
     int quickStampIndex() const;
