@@ -26,6 +26,10 @@ class QAction;
 class QActionGroup;
 
 namespace Tiled {
+
+class Tile;
+class ObjectTemplate;
+
 namespace Internal {
 
 class AbstractTool;
@@ -44,7 +48,7 @@ class ToolManager : public QObject
 
 public:
     ToolManager(QObject *parent = nullptr);
-    ~ToolManager();
+    ~ToolManager() override;
 
     void setMapDocument(MapDocument *mapDocument);
 
@@ -55,6 +59,16 @@ public:
 
     void retranslateTools();
 
+    void createShortcuts(QWidget *parent);
+
+public slots:
+    /**
+     * Sets the tile that will be used when the creation mode is
+     * CreateTileObjects or when replacing a tile of a tile object.
+     */
+    void setTile(Tile *tile);
+    void setObjectTemplate(ObjectTemplate *objectTemplate);
+
 signals:
     void selectedToolChanged(AbstractTool *tool);
 
@@ -80,6 +94,8 @@ private:
     AbstractTool *mDisabledTool;
     AbstractTool *mPreviouslyDisabledTool;
     MapDocument *mMapDocument;
+    Tile *mTile;
+    ObjectTemplate *mObjectTemplate;
 
     bool mSelectEnabledToolPending;
 };
